@@ -42,11 +42,9 @@ class Interpolator:
         :type unc_methods: list(str) (optional)
         :param unc_methods_hr: interpolation methods to use in the calculation of the model error for interpolation between high resolution measurements. Not used for gpr. Defaults to None, in which case a standard list is used for each interpolation method.
         :type unc_methods_hr: list(str) (optional)
-        :param add_model_error:
-        :type add_model_error:
         :param min_scale: minimum bound on the scale parameter in the gaussian process regression. Only used if gpr is selected as method. Defaults to 0.3
         :type min_scale: float (optional)
-        :param extrapolate: extrpolation method, which can be set to "nearest" (in which case nearest values are used for extrapolation) or "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), defaults to "extrapolate".
+        :param extrapolate: extrapolation method, which can be set to "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), "nearest" (in which case nearest values are used for extrapolation), or "linear" (in which case linear extrapolation is used). Defaults to "extrapolate".
         :type extrapolate: str (optional)
         :param add_model_error: Boolean to indicate whether model error should be added to interpolated values to account for interpolation errors (useful in Monte Carlo approaches). Defaults to False
         :type add_model_error: bool (optional)
@@ -173,7 +171,7 @@ def interpolate_1d(
     x_i,
     y_i,
     x,
-    method="cubic",
+    method="linear",
     unc_methods=None,
     u_y_i=None,
     corr_y_i=None,
@@ -205,7 +203,7 @@ def interpolate_1d(
     :type corr_y_i: np.ndarray or str (optional)
     :param min_scale: minimum bound on the scale parameter in the gaussian process regression. Only used if gpr is selected as method. Defaults to 0.3
     :type min_scale: float (optional)
-    :param extrapolate: extrpolation method, which can be set to "nearest" (in which case nearest values are used for extrapolation) or "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), defaults to "extrapolate".
+    :param extrapolate: extrapolation method, which can be set to "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), "nearest" (in which case nearest values are used for extrapolation), or "linear" (in which case linear extrapolation is used). Defaults to "extrapolate".
     :type extrapolate: str (optional)
     :param return_uncertainties: Boolean to indicate whether interpolation uncertainties should be calculated and returned. Defaults to False
     :type return_uncertainties: bool (optional)
@@ -238,6 +236,21 @@ def interpolate_1d(
             parallel_cores=parallel_cores,
         )
 
+    elif add_model_error:
+        if unc_methods is None:
+            unc_methods = default_unc_methods(method)
+
+        extrapolate_methods = [extrapolate, "nearest"]
+        return interpolate_1d(
+            x_i,
+            y_i,
+            x,
+            method=np.random.choice(unc_methods, 1)[0],
+            extrapolate=np.random.choice(extrapolate_methods, 1)[0],
+            return_uncertainties=False,
+            add_model_error=False,
+        )
+
     if method.lower() in [
         "linear",
         "nearest",
@@ -249,27 +262,11 @@ def interpolate_1d(
         "previous",
         "next",
     ]:
-        if extrapolate == "nearest":
-            f_i = interp1d(
-                x_i,
-                y_i,
-                kind=method.lower(),
-                fill_value=(y_i[0], y_i[-1]),
-                bounds_error=False,
-            )
-        elif extrapolate == "extrapolate":
-            f_i = interp1d(x_i, y_i, kind=method.lower(), fill_value="extrapolate")
-        else:
-            f_i = interp1d(x_i, y_i, kind=method.lower())
-
+        f_i = interp1d(x_i, y_i, kind=method.lower(), fill_value="extrapolate")
         y = f_i(x).squeeze()
 
     elif method.lower() == "ius":
-        if extrapolate == "nearest":
-            ext = 3
-        else:
-            ext = 0
-        f_i = InterpolatedUnivariateSpline(x_i, y_i, ext=ext)
+        f_i = InterpolatedUnivariateSpline(x_i, y_i, ext=0)
         y = f_i(x).squeeze()
 
     elif method.lower() == "lagrange":
@@ -282,58 +279,59 @@ def interpolate_1d(
             % (method)
         )
 
-    if (not return_corr) and (not return_uncertainties) and (not add_model_error):
+    y = redo_extrapolation(x_i, y_i, x, y, extrapolate)
+
+    if (not return_corr) and (not return_uncertainties):
         return y
 
     else:
         if unc_methods is None:
-            if method.lower() in ["nearest", "previous", "next"]:
-                unc_methods = ["nearest", "previous", "next", "linear"]
-            elif method.lower() == "linear":
-                unc_methods = ["linear", "quadratic", "cubic"]
-            elif method.lower() == "quadratic":
-                unc_methods = ["linear", "quadratic", "cubic"]
-            elif method.lower() == "cubic":
-                unc_methods = ["linear", "quadratic", "cubic"]
-            elif method.lower() == "lagrange":
-                unc_methods = ["lagrange", "linear", "cubic"]
-            elif method.lower() == "ius":
-                unc_methods = ["ius", "linear", "cubic"]
-            else:
-                raise ValueError(
-                    "comet_maths.interpolation: uncertainties for the model error for this interpolation method (%s) are not yet implemented"
-                    % (method)
-                )
+            unc_methods = default_unc_methods(method)
 
-        if add_model_error or include_model_uncertainties:
+        if include_model_uncertainties:
             u_y_model, corr_y_model, cov_model = model_error_analytical_methods(
                 x_i, y_i, x, unc_methods=unc_methods
             )
 
-        if add_model_error:
-            # y=cm.generate_sample(1,y,u_y_model,corr_y_model)
-            y = cm.generate_sample_cov(1, y, cov_model, diff=0.1).squeeze()
-
         if (not return_uncertainties) and (not return_corr):
             return y
 
         if u_y_i is None:
-            y_unc = u_y_model
-            y_corr = corr_y_model
+            if include_model_uncertainties:
+                u_y_model, corr_y_model, cov_model = model_error_analytical_methods(
+                    x_i, y_i, x, unc_methods=unc_methods
+                )
+                y_unc = u_y_model
+                y_corr = corr_y_model
+
+            else:
+                y_unc = None
+                y_corr = None
 
         else:
             prop = punpy.MCPropagation(MCsteps, parallel_cores=1)
             intp = Interpolator(
                 unc_methods=unc_methods, add_model_error=include_model_uncertainties
             )
-            y_unc, y_corr = prop.propagate_random(
-                intp.interpolate_1d,
-                [x_i, y_i, x],
-                [None, u_y_i, None],
-                corr_x=[None, corr_y_i, None],
-                return_corr=True,
-            )
-            y_corr[np.where(np.isnan(y_corr))] = 0
+
+            if return_corr:
+                y_unc, y_corr = prop.propagate_random(
+                    intp.interpolate_1d,
+                    [x_i, y_i, x],
+                    [None, u_y_i, None],
+                    corr_x=[None, corr_y_i, None],
+                    return_corr=return_corr,
+                )
+                y_corr[np.where(np.isnan(y_corr))] = 0
+
+            else:
+                y_unc = prop.propagate_random(
+                    intp.interpolate_1d,
+                    [x_i, y_i, x],
+                    [None, u_y_i, None],
+                    corr_x=[None, corr_y_i, None],
+                    return_corr=return_corr,
+                )
 
         if return_uncertainties and return_corr:
             return y, y_unc, y_corr
@@ -341,6 +339,64 @@ def interpolate_1d(
             return y, y_unc
 
 
+def default_unc_methods(method):
+    """
+    Function providing for each analytical interpolation method, the default methods that are compared to determine the model uncertainty for this interpolation method.
+
+    :param method: method used in the interpolation
+    :type method: str
+    :return: methods used to determine the model uncertainty on the provided method
+    :rtype: list(str)
+    """
+    if method.lower() in ["nearest", "previous", "next"]:
+        unc_methods = ["nearest", "previous", "next", "linear"]
+    elif method.lower() == "linear":
+        unc_methods = ["linear", "quadratic", "cubic"]
+    elif method.lower() == "quadratic":
+        unc_methods = ["linear", "quadratic", "cubic"]
+    elif method.lower() == "cubic":
+        unc_methods = ["linear", "quadratic", "cubic"]
+    elif method.lower() == "lagrange":
+        unc_methods = ["lagrange", "linear", "cubic"]
+    elif method.lower() == "ius":
+        unc_methods = ["ius", "linear", "cubic"]
+    else:
+        raise ValueError(
+            "comet_maths.interpolation: uncertainties for the model error for this interpolation method (%s) are not yet implemented"
+            % (method)
+        )
+    return unc_methods
+
+
+def redo_extrapolation(x_i, y_i, x, y, extrapolate):
+    """
+    function to check if extrapolate is "nearest" or "linear, and if so, redo the extrapolation
+
+    :param x_i: Independent variable quantity x (coordinate data of y_i)
+    :type x_i: np.ndarray
+    :param y_i: measured variable quantity y (data to interpolate)
+    :type y_i: np.ndarray
+    :param x: Independent variable quantity x for which we are trying to obtain the measurand y
+    :type x: np.ndarray
+    :param y: interpolated values using standard method
+    :type y: np.ndarray
+    :param extrapolate: extrapolation method, which can be set to "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), "nearest" (in which case nearest values are used for extrapolation), or "linear" (in which case linear extrapolation is used). Defaults to "extrapolate".
+    :type extrapolate: str (optional)
+    :return: interpolated values with correct extrapolation
+    :rtype: np.ndarray
+    """
+    if extrapolate == "nearest":
+        y[x < x_i[0]] = y_i[0]
+        y[x > x_i[-1]] = y_i[-1]
+
+    elif extrapolate == "linear":
+        f_lin = interp1d(x_i, y_i, kind="linear", fill_value="extrapolate")
+        y[x < x_i[0]] = f_lin(x[x < x_i[0]])
+        y[x > x_i[-1]] = f_lin(x[x > x_i[-1]])
+
+    return y
+
+
 def model_error_analytical_methods(
     x_i, y_i, x, unc_methods=["linear", "quadratic", "cubic"]
 ):
@@ -415,7 +471,7 @@ def gaussian_process_regression(
     :type min_scale: float (optional)
     :param max_scale: maximum bound on the scale parameter in the gaussian process regression. Defaults to 100
     :type max_scale: float (optional)
-    :param extrapolate: extrpolation method, which can be set to "nearest" (in which case nearest values are used for extrapolation) or "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), defaults to "extrapolate".
+    :param extrapolate: extrapolation method, which can be set to "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), "nearest" (in which case nearest values are used for extrapolation), or "linear" (in which case linear extrapolation is used). Defaults to "extrapolate".
     :type extrapolate: str (optional)
     :param return_uncertainties: Boolean to indicate whether interpolation uncertainties should be calculated and returned. Defaults to False
     :type return_uncertainties: bool (optional)
@@ -437,6 +493,8 @@ def gaussian_process_regression(
         x_i, y_i, x, kernel=kernel, min_scale=min_scale, max_scale=max_scale
     )
 
+    y_out = redo_extrapolation(x_i, y_i, x, y_out, extrapolate)
+
     if add_model_error:
         y_out = cm.generate_sample_cov(1, y_out, cov_model, diff=0.1).squeeze()
 
@@ -447,10 +505,6 @@ def gaussian_process_regression(
         corr_y_out = cm.correlation_from_covariance(cov_model)
         u_y_out = cm.uncertainty_from_covariance(cov_model)
 
-    if extrapolate == "nearest":
-        y_out[x < x_i[0]] = y_i[0]
-        y_out[x > x_i[-1]] = y_i[-1]
-
     else:
         # next determine if a simple uncertainty from gpr is possible or if MC is necessary
         uncertainties_simple = False
@@ -521,8 +575,6 @@ def gpr_basics(
     :type min_scale: float (optional)
     :param max_scale: maximum bound on the scale parameter in the gaussian process regression. Defaults to 100
     :type max_scale: float (optional)
-    :param return_cov: Boolean to indicate whether the covariance matrix for interpolation uncertainties should be calculated and returned. Defaults to False
-    :type return_cov:  bool (optional)
     :return: The measurand y evaluated at the values x (interpolated data)
     :rtype: np.ndarray
     """
@@ -568,8 +620,8 @@ def interpolate_1d_along_example(
     y_hr,
     x,
     relative=True,
-    method="gpr",
-    method_hr="cubic",
+    method="linear",
+    method_hr="linear",
     unc_methods=None,
     unc_methods_hr=None,
     u_y_i=None,
@@ -622,7 +674,7 @@ def interpolate_1d_along_example(
     :type corr_y_hr: np.ndarray or str (optional)
     :param min_scale: minimum bound on the scale parameter in the gaussian process regression. Only used if gpr is selected as method. Defaults to 0.3
     :type min_scale: float (optional)
-    :param extrapolate: extrpolation method, which can be set to "nearest" (in which case nearest values are used for extrapolation) or "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), defaults to "extrapolate".
+    :param extrapolate: extrapolation method, which can be set to "extrapolate" (in which case extrapolation is used using interpolation method defined in "method"), "nearest" (in which case nearest values are used for extrapolation), or "linear" (in which case linear extrapolation is used). Defaults to "extrapolate".
     :type extrapolate: str (optional)
     :param return_uncertainties: Boolean to indicate whether interpolation uncertainties should be calculated and returned. Defaults to False
     :type return_uncertainties: bool (optional)
@@ -652,7 +704,7 @@ def interpolate_1d_along_example(
         add_model_error=add_model_error,
     )
 
-    if x == x_hr:
+    if np.all(x == x_hr):
         y_hr_out = y_hr
     else:
         y_hr_out = interpolate_1d(
@@ -699,142 +751,6 @@ def interpolate_1d_along_example(
     if (not return_corr) and (not return_uncertainties):
         return y_out
 
-    # if return_uncertainties:
-    #     y_hr_i, u_y_hr_i, corr_y_hr_i = interpolate_1d(
-    #         x_hr,
-    #         y_hr,
-    #         x_i,
-    #         method=method_hr,
-    #         u_y_i=u_y_hr,
-    #         corr_y_i=corr_y_hr,
-    #         unc_methods=unc_methods_hr,
-    #         min_scale=min_scale,
-    #         return_uncertainties=return_uncertainties,
-    #         return_corr=True,
-    #         add_model_error=add_model_error,
-    #         include_model_uncertainties=include_model_uncertainties,
-    #         extrapolate=extrapolate,
-    #         MCsteps=MCsteps,
-    #         parallel_cores=parallel_cores,
-    #     )
-    #     cov_y_hr_i = cm.convert_corr_to_cov(corr_y_hr_i, u_y_hr_i)
-    #     if x == x_hr:
-    #         y_hr_out = y_hr
-    #     else:
-    #         y_hr_out, u_y_hr_out, corr_y_hr_out = interpolate_1d(
-    #             x_hr,
-    #             y_hr,
-    #             x,
-    #             method=method_hr,
-    #             u_y_i=u_y_hr,
-    #             corr_y_i=corr_y_hr,
-    #             unc_methods=unc_methods_hr,
-    #             min_scale=min_scale,
-    #             return_uncertainties=return_uncertainties,
-    #             return_corr=True,
-    #             add_model_error=add_model_error,
-    #             include_model_uncertainties=include_model_uncertainties,
-    #             extrapolate=extrapolate,
-    #             MCsteps=MCsteps,
-    #             parallel_cores=parallel_cores,
-    #         )
-    #         cov_y_hr_out = cm.convert_corr_to_cov(corr_y_hr_out, u_y_hr_out)
-    #
-    # # if x == x_hr:
-    #     #     y_hr_out2 = y_hr
-    #     # else:
-    #     #     y_hr_out2 = interpolate_1d(
-    #     #         x_hr,
-    #     #         y_hr,
-    #     #         x,
-    #     #         method=method_hr,
-    #     #         min_scale=min_scale,
-    #     #         extrapolate=extrapolate,
-    #     #         include_model_uncertainties=include_model_uncertainties,
-    #     #     )
-    #     if relative:
-    #         y_norm_i = y_i / y_hr_i
-    #         u_y_norm_i = (
-    #             (u_y_i / y_i) ** 2 + (u_y_hr_i / y_hr_i) ** 2
-    #         ) ** 0.5 * y_norm_i
-    #         u_y_norm_i[np.where(y_norm_i==0.)]=0.
-    #     else:
-    #         y_norm_i = y_i - y_hr_i
-    #         u_y_norm_i = ((u_y_i) ** 2 + (u_y_hr_i) ** 2) ** 0.5
-    #         if corr_y_i=="rand":
-    #             corr_y_i=np.eye(len(u_y_i))
-    #         if corr_y_i=="syst":
-    #             corr_y_i=np.ones((len(u_y_i),len(y_i)))
-    #
-    #         cov_y_i = cm.convert_corr_to_cov(corr_y_i, u_y_i)
-    #
-    #         cov_y_norm_i = cov_y_i + cov_y_hr_i
-    #         u_y_norm_i2 = cm.uncertainty_from_covariance(cov_y_norm_i)
-    #         corr_y_norm_i = cm.correlation_from_covariance(cov_y_norm_i)
-    #
-    #     y_norm_hr, u_y_norm_hr = interpolate_1d(
-    #         x_i,
-    #         y_norm_i,
-    #         x,
-    #         method=method,
-    #         u_y_i=u_y_norm_i,
-    #         corr_y_i=corr_y_norm_i,
-    #         min_scale=min_scale,
-    #         return_uncertainties=return_uncertainties,
-    #         unc_methods=unc_methods,
-    #         extrapolate=extrapolate,
-    #         MCsteps=MCsteps,
-    #         parallel_cores=parallel_cores,
-    #         include_model_uncertainties=include_model_uncertainties,
-    #     )
-    #     y_norm_hr = y_norm_hr.squeeze()
-    #
-    #     if relative:
-    #         r = -u_y_hr_out / u_y_norm_hr
-    #         y_out = y_norm_hr * y_hr_out
-    #         var=(
-    #                 u_y_hr_out / y_hr_out
-    #                 + 2 * r * u_y_norm_hr / y_norm_hr * u_y_hr_out / y_hr_out
-    #         )** 2
-    #         u_y_out = (
-    #             (u_y_norm_hr / y_norm_hr) ** 2
-    #             + (
-    #                 u_y_hr_out / y_hr_out
-    #                 + 2 * r * u_y_norm_hr / y_norm_hr * u_y_hr_out / y_hr_out
-    #             )
-    #             ** 2
-    #         ) ** 0.5 * y_out
-    #     else:
-    #         r = -u_y_hr_out / u_y_norm_hr
-    #         y_out = y_norm_hr + y_hr_out
-    #         u_y_out = (
-    #             (u_y_norm_hr) ** 2
-    #             - (u_y_hr_out) ** 2
-    #
-    #         ) ** 0.5
-    #
-    #     if plot_residuals:
-    #         plt.plot(x_i, y_norm_i, "ro", label="low-res residuals")
-    #         plt.plot(x, y_norm_hr, "g-", label="high-res residuals")
-    #         plt.fill_between(
-    #             x,
-    #             y_norm_hr - 1.9600 * u_y_norm_hr,
-    #             (y_norm_hr + 1.9600 * u_y_norm_hr),
-    #             alpha=0.25,
-    #             fc="g",
-    #             ec="None",
-    #             label="95% confidence interval",
-    #             lw=0,
-    #         )
-    #
-    #         plt.ylabel("Residuals")
-    #         plt.xlabel("x")
-    #         plt.legend()
-    #         plt.savefig("residuals.png")
-    #         plt.clf()
-    #
-    # else:
-
     else:
         prop = punpy.MCPropagation(MCsteps, parallel_cores=parallel_cores)
         intp = Interpolator(
